@@ -256,45 +256,52 @@ def read_customer(ref_code):
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)
     
 @customer_api_bp.route('/customer-key/<customer_key>', methods=['GET'])
-@auth_token_required
-@device_is_activated
+#@auth_token_required
+#@device_is_activated
 def read_customer_by_customer_key(customer_key):
     
     logger.debug('customer_key=%s', customer_key)
     
     if is_not_empty(customer_key):
+        customer_details_dict = None
         db_client = create_db_client(caller_info="read_customer_by_customer_key")
         
         with db_client.context():
             customer = Customer.fetch(customer_key)
             
-        
-        if customer:
-            '''
-            customer_property_list = ['name', 'email', 'mobile_phone', 'gender', 'birth_date', 'merchant_reference_code']
-            
-            response_data = {
-                        'customer_key'              : customer.key_in_str,
-                        'registered_datetime'       : customer.registered_datetime.strftime("%d-%m-%Y %H:%M:%S"),
-                        'merchant_reference_code'   : customer.merchant_reference_code,
-                        'merchant_account_key'      : customer.registered_merchant_acct_key,
-                        'outlet_key'                : customer.registered_outlet_key,  
-                        'user_details'              : user_details_dict(user_details_dict), 
-                        }
-            '''
-            
-            user_details = customer.registered_user_acct
-            customer_details_dict = customer.to_dict(date_format="%d-%m-%Y", datetime_format="%d-%m-%Y %H:%M:%S")
-            customer_details_dict['customer_key']               = customer.key_in_str
-            customer_details_dict['is_email_verified']          = user_details.is_email_verified
-            customer_details_dict['is_mobile_phone_verified']   = user_details.is_mobile_phone_verified
-            
-            if 'entitled_voucher_summary' in customer_details_dict:
-                customer_details_dict['voucher_summary']            = customer.entitled_voucher_summary
-                del customer_details_dict['entitled_voucher_summary']
+            if customer:
+                user_details = customer.registered_user_acct
+                customer_details_dict = customer.to_dict(
+                                                    date_format="%d-%m-%Y", 
+                                                    datetime_format="%d-%m-%Y %H:%M:%S",
+                                                    excluded_dict_properties = [
+                                                            'registered_merchant_acct', 
+                                                            'memberships_list', 
+                                                            'registered_merchant_acct_key',
+                                                            'tier_membership_key',
+                                                            
+                                                            ],
+                                                    )
+                customer_details_dict['customer_key']               = customer.key_in_str
+                customer_details_dict['is_email_verified']          = user_details.is_email_verified
+                customer_details_dict['is_mobile_phone_verified']   = user_details.is_mobile_phone_verified
+                
+                
+                customer_basic_memberships_list = _list_customer_basic_memberships(customer)
+                if customer_basic_memberships_list:
+                    customer_details_dict['basic_memberships'] = customer_basic_memberships_list
+                
+                customer_tier_membership = _get_tier_membership(customer)
+                if customer_tier_membership:
+                    customer_details_dict['tier_membership']  = customer_tier_membership
+                
+                if 'entitled_voucher_summary' in customer_details_dict:
+                    customer_details_dict['voucher_summary']            = customer.entitled_voucher_summary
+                    del customer_details_dict['entitled_voucher_summary']
             
+        if customer_details_dict:
             return jsonify(customer_details_dict)
-            #return jsonify(response_data)
+            
             
         else:
             logger.warn('Customer with customer_key (%s) is not found', customer_key)
@@ -348,7 +355,7 @@ def update_customer(customer_key):
                         
                         if checking_user_acct:
                             is_email_used = True
-                            is_used_email_same_user_acct = checking_user_acct.key_in_str == updating_customer.registered_user_acct_key
+                            is_used_email_same_user_acct = checking_user_acct.reference_code == updating_customer.reference_code
                         
                 if is_customer_mobile_phone_changed:
                     if is_not_empty(mobile_phone):
@@ -356,7 +363,7 @@ def update_customer(customer_key):
                     
                         if checking_user_acct:
                             is_mobile_phone_used = True
-                            is_used_mobile_phone_same_user_acct = checking_user_acct.key_in_str == updating_customer.registered_user_acct_key
+                            is_used_mobile_phone_same_user_acct = checking_user_acct.reference_code == updating_customer.reference_code
                                 
                 logger.debug('is_customer_email_changed=%s', is_customer_email_changed)
                 logger.debug('is_customer_mobile_phone_changed=%s', is_customer_mobile_phone_changed)
@@ -567,6 +574,7 @@ def search_customer(limit):
         email                       = search_customer_form.email.data
         reference_code              = search_customer_form.reference_code.data
         merchant_reference_code     = search_customer_form.merchant_reference_code.data
+        customer_data               = search_customer_form.customer_data.data
     
         acct_id     = request.headers.get('x-acct-id')
         
@@ -601,6 +609,8 @@ def search_customer(limit):
                 customer_details_dict['is_email_verified']          = user_details.is_email_verified
                 customer_details_dict['is_mobile_phone_verified']   = user_details.is_mobile_phone_verified
                 
+                #if 'reward' in customer_data:
+                
                 if 'entitled_voucher_summary' in customer_details_dict:
                     customer_details_dict['voucher_summary']            = c.entitled_voucher_summary
                     #del customer_details_dict['entitled_voucher_summary'] 
@@ -608,6 +618,15 @@ def search_customer(limit):
                 
                 logger.debug('customer_details_dict=%s', customer_details_dict)
                 
+                #if 'membership' in customer_data
+                customer_basic_memberships_list = _list_customer_basic_memberships(c)
+                if customer_basic_memberships_list:
+                    customer_details_dict['basic_memberships'] = customer_basic_memberships_list
+                
+                tier_membership_data = _get_tier_membership(c)
+                if tier_membership_data:
+                    customer_details_dict['tier_membership'] = tier_membership_data
+                
                 customer_list.append(customer_details_dict)
     
         #return create_rest_message(status_code=customer_list.OK, customer_list=customer_list)
@@ -623,6 +642,53 @@ def search_customer(limit):
         #return create_rest_message(gettext('Invalid input'), status_code=StatusCode.BAD_REQUEST)
         #return create_rest_message('Invalid input', status_code=StatusCode.BAD_REQUEST)
 
+
+def _list_customer_basic_memberships(customer):
+    customer_membership_final_list = []
+    
+    if is_not_empty(customer.memberships_list):
+        customer_memberships_list = CustomerMembership.list_all_by_customer(customer)
+        if is_not_empty(customer_memberships_list):
+            merchant_acct = customer.registered_merchant_acct
+            merchant_memberships_list = MerchantMembership.list_by_merchant_acct(merchant_acct)
+            
+            for cm in customer_memberships_list:
+                for mm in merchant_memberships_list:
+                    logger.debug('cm=%s', cm)
+                    if mm.key_in_str == cm.merchant_membership_key:
+                        membership_data = {
+                                                        'key'           : mm.key_in_str,
+                                                        'label'         : mm.label,
+                                                        'entitled_date' : cm.entitled_date.strftime('%d-%m-%Y'),
+                                                        'expiry_date'   : cm.expiry_date.strftime('%d-%m-%Y'),
+                                                        'is_tier'       : False,
+                                                        }
+                        
+                        if cm.renewed_date is not None:
+                            membership_data['renewed_date'] = cm.renewed_date.strftime('%d-%m-%Y'),
+                        
+                        customer_membership_final_list.append(membership_data)
+                        break
+    return customer_membership_final_list
+
+def _get_tier_membership(customer):
+            
+    if is_not_empty(customer.tier_membership):
+        merchant_tier_membership = customer.tier_membership_entity
+        customer_tier_membership = CustomerTierMembership.get_by_customer_and_merchant_tier_membership(customer, merchant_tier_membership)
+        
+        membership_data = {
+                        'key'           : customer_tier_membership.merchant_tier_membership_key,
+                        'label'         : merchant_tier_membership.label,
+                        'entitled_date' : customer_tier_membership.entitled_date.strftime('%d-%m-%Y'),
+                        'expiry_date'   : customer_tier_membership.expiry_date.strftime('%d-%m-%Y'),
+                        'is_tier'       : True,
+                        }
+        
+        return membership_data
+        
+    
+
 @customer_api_bp.route('/reference-code/<reference_code>/list-membership', methods=['GET'])
 @auth_token_required
 #@test_session_expired
@@ -630,59 +696,20 @@ def list_customer_membership(reference_code):
     if is_not_empty(reference_code):
         acct_id   = request.headers.get('x-acct-id')
         db_client = create_db_client(caller_info="list_customer_membership")
-        customer_membership_final_list = []
+        customer_membershis_list = []
         
         logger.debug('reference_code=%s', reference_code)
         
         with db_client.context():
-            merchant_acct = MerchantAcct.fetch(acct_id)
-            customer = Customer.get_by_reference_code(reference_code, merchant_acct)
-            customer_memberhips_list        = CustomerMembership.list_active_by_customer(customer)
-            customer_tier_memberhips_list   = CustomerTierMembership.list_active_by_customer(customer)
+            merchant_acct   = MerchantAcct.fetch(acct_id)
+            customer        = Customer.get_by_reference_code(reference_code, merchant_acct)
             
-            logger.debug('customer=%s', customer)
-            
-            logger.debug('customer_memberhips_list=%s', customer_memberhips_list)
-            
-            if is_not_empty(customer_memberhips_list):
-                merchant_memberships_list = MerchantMembership.list_by_merchant_acct(merchant_acct)
-                
-                for cm in customer_memberhips_list:
-                    for mm in merchant_memberships_list:
-                        if mm.key_in_str == cm.merchant_membership_key:
-                            customer_membership_data = {
-                                                            'key'           : cm.key_in_str,
-                                                            'label'         : mm.label,
-                                                            'entitled_date' : cm.entitled_date.strftime('%d-%m-%Y'),
-                                                            'expiry_date'   : cm.expiry_date.strftime('%d-%m-%Y'),
-                                                            'is_tier'       : False,
-                                                            }
-                            
-                            if cm.renewed_date is not None:
-                                customer_membership_data['renewed_date'] = cm.renewed_date.strftime('%d-%m-%Y'),
-                            
-                            customer_membership_final_list.append(customer_membership_data)
-                            break
-                        
-            if is_not_empty(customer_tier_memberhips_list):
-                merchant_tier_memberships_list = MerchantTierMembership.list_by_merchant_acct(merchant_acct)
-                
-                for cm in customer_tier_memberhips_list:
-                    for mm in merchant_tier_memberships_list:
-                        if mm.key_in_str == cm.merchant_tier_membership_key:
-                            customer_membership_data = {
-                                                            'key'           : cm.key_in_str,
-                                                            'label'         : mm.label,
-                                                            'entitled_date' : cm.entitled_date.strftime('%d-%m-%Y'),
-                                                            'expiry_date'   : cm.expiry_date.strftime('%d-%m-%Y'),
-                                                            'is_tier'       : True,
-                                                            }
-                            
-                            customer_membership_final_list.append(customer_membership_data)
-                            break            
-        
+            customer_membershis_list = _list_customer_basic_memberships(customer)
+            tier_membership_data = _get_tier_membership(customer)
+            if tier_membership_data:
+                customer_membershis_list.append(tier_membership_data)
     
-        return jsonify(customer_membership_final_list)
+        return jsonify(customer_membershis_list)
     else:
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)    
                 
