@@ -13,7 +13,7 @@ from trexapi import conf
 from flask_restful import Api
 from trexmodel.utils.model.model_util import create_db_client
 from trexmodel.models.datastore.merchant_models import MerchantUser,\
-    MerchantAcct
+    MerchantAcct, Outlet
 import hashlib
 from random import random
 from trexlib.utils.string_util import is_not_empty
@@ -25,7 +25,10 @@ from trexmodel.models.datastore.pos_models import POSSetting
 from trexapi.controllers.api_routes import APIBaseResource
 from trexmodel.models import merchant_helpers
 from trexadmin.libs.http import create_rest_message, StatusCode
-from trexmodel.models.datastore.customer_models import Customer
+from trexmodel.models.datastore.customer_models import Customer,\
+    CustomerMembership, CustomerTierMembership
+from trexmodel.models.datastore.rating_models import MerchantRatingResult
+from trexmodel.models.datastore.membership_models import MerchantMembership
 
 #logger = logging.getLogger('api')
 logger = logging.getLogger('debug')
@@ -85,15 +88,111 @@ def read_merchant_acct(merchant_acct_code):
     if is_not_empty(merchant_acct_code):
         db_client = create_db_client(caller_info="read_merchant_acct")
         with db_client.context():
-            merchant_acct = MerchantAcct.get_by_account_code(merchant_acct_code)
+            merchant_acct   = MerchantAcct.get_by_account_code(merchant_acct_code)
+            
             logger.debug('website=%s', merchant_acct.website)
             merchant_info = merchant_helpers.construct_merchant_acct_info(merchant_acct) 
+            merchant_rating_result  = MerchantRatingResult.get_by_merchant_acct(merchant_acct)
+            
+            
+            
+            if merchant_rating_result:
+                logger.debug('rating_result=%s', merchant_rating_result.rating_result)
+                merchant_info['rating_reviews'] = merchant_rating_result.rating_result
+            
+                
+        logger.debug('merchant_info=%s', merchant_info)
             
         return create_rest_message(status_code=StatusCode.OK,
                                                **merchant_info
                                                )
     else:
         return create_rest_message("Missing merchant account key", status_code=StatusCode.BAD_REQUEST)
+    
+@merchant_api_bp.route('/<merchant_acct_code>/joined-brand-details/reference-code/<reference_code>', methods=['GET'])
+def read_joined_brand_details(merchant_acct_code, reference_code):
+    
+    merchant_acct           = None
+    
+    if is_not_empty(merchant_acct_code) and is_not_empty(reference_code):    
+        db_client = create_db_client(caller_info="read_merchant_acct")
+        with db_client.context():
+            merchant_acct           = MerchantAcct.get_by_account_code(merchant_acct_code)
+            customer                = Customer.get_by_reference_code(reference_code, merchant_acct)
+            
+            merchant_info           = merchant_helpers.construct_merchant_acct_info(merchant_acct) 
+            merchant_rating_result  = MerchantRatingResult.get_by_merchant_acct(merchant_acct)
+            
+            if merchant_rating_result:
+                logger.debug('rating_result=%s', merchant_rating_result.rating_result)
+                merchant_info['rating_reviews'] = merchant_rating_result.rating_result
+            
+            user_details                                        = customer.registered_user_acct
+            customer_details = customer.to_dict(date_format="%d-%m-%Y", datetime_format="%d-%m-%Y %H:%M:%S")
+            customer_details['customer_key']               = customer.key_in_str
+            customer_details['is_email_verified']          = user_details.is_email_verified
+            customer_details['is_mobile_phone_verified']   = user_details.is_mobile_phone_verified
+            
+            customer_basic_memberships_list = _list_customer_basic_memberships(customer)
+            if customer_basic_memberships_list:
+                customer_details['basic_memberships'] = customer_basic_memberships_list
+            
+            tier_membership_data = _get_tier_membership(customer)
+            if tier_membership_data:
+                customer_details['tier_membership'] = tier_membership_data
+                
+        logger.debug('merchant_info=%s', merchant_info)
+            
+        return create_rest_message(status_code=StatusCode.OK,
+                                               merchant_info    = merchant_info,
+                                               customer_details = customer_details,
+                                               )
+    else:
+        return create_rest_message("Missing merchant account key", status_code=StatusCode.BAD_REQUEST)    
+
+def _list_customer_basic_memberships(customer):
+    customer_membership_final_list = []
+    
+    if is_not_empty(customer.memberships_list):
+        customer_memberships_list = CustomerMembership.list_all_by_customer(customer)
+        if is_not_empty(customer_memberships_list):
+            merchant_acct = customer.registered_merchant_acct
+            merchant_memberships_list = MerchantMembership.list_by_merchant_acct(merchant_acct)
+            
+            for cm in customer_memberships_list:
+                for mm in merchant_memberships_list:
+                    logger.debug('cm=%s', cm)
+                    if mm.key_in_str == cm.merchant_membership_key:
+                        membership_data = {
+                                                        'key'           : mm.key_in_str,
+                                                        'label'         : mm.label,
+                                                        'entitled_date' : cm.entitled_date.strftime('%d-%m-%Y'),
+                                                        'expiry_date'   : cm.expiry_date.strftime('%d-%m-%Y'),
+                                                        'is_tier'       : False,
+                                                        }
+                        
+                        if cm.renewed_date is not None:
+                            membership_data['renewed_date'] = cm.renewed_date.strftime('%d-%m-%Y'),
+                        
+                        customer_membership_final_list.append(membership_data)
+                        break
+    return customer_membership_final_list
+
+def _get_tier_membership(customer):
+            
+    if is_not_empty(customer.tier_membership):
+        merchant_tier_membership = customer.tier_membership_entity
+        customer_tier_membership = CustomerTierMembership.get_by_customer_and_merchant_tier_membership(customer, merchant_tier_membership)
+        
+        membership_data = {
+                        'key'           : customer_tier_membership.merchant_tier_membership_key,
+                        'label'         : merchant_tier_membership.label,
+                        'entitled_date' : customer_tier_membership.entitled_date.strftime('%d-%m-%Y'),
+                        'expiry_date'   : customer_tier_membership.expiry_date.strftime('%d-%m-%Y'),
+                        'is_tier'       : True,
+                        }
+        
+        return membership_data
 
 @merchant_api_bp.route('/<merchant_key>/customer-details/reference-code/<reference_code>', methods=['GET'])
 def read_merchant_customer_details(merchant_key, reference_code):
@@ -114,6 +213,15 @@ def read_merchant_customer_details(merchant_key, reference_code):
                 customer_details_dict['customer_key']               = customer.key_in_str
                 customer_details_dict['is_email_verified']          = user_details.is_email_verified
                 customer_details_dict['is_mobile_phone_verified']   = user_details.is_mobile_phone_verified
+                
+                customer_basic_memberships_list = _list_customer_basic_memberships(customer)
+                if customer_basic_memberships_list:
+                    customer_details_dict['basic_memberships'] = customer_basic_memberships_list
+                
+                tier_membership_data = _get_tier_membership(customer)
+                if tier_membership_data:
+                    customer_details_dict['tier_membership'] = tier_membership_data
+                
                 '''
                 if 'entitled_voucher_summary' in customer_details_dict:
                     customer_details_dict['voucher_summary']            = customer.entitled_voucher_summary
@@ -268,9 +376,11 @@ class ProgramDeviceAuthenticate(AccountActivatedAPIResource):
             data_in_json        = request.get_json()
             activation_code     = data_in_json.get('activation_code')
             device_setting      = LoyaltyDeviceSetting.get_by_activation_code(activation_code)
+            access_outlet       = Outlet.fetch(outlet_key)
             assigned_outlet_key = device_setting.assigned_outlet_key
             
             logger.debug('device assigned outlet name=%s', device_setting.assigned_outlet_entity.name)
+            logger.debug('access outlet name=%s', access_outlet.name)
             
             if outlet_key == device_setting.assigned_outlet_key:
                 logger.debug('login outlet is same as device setting assigned outlet')
@@ -296,9 +406,11 @@ class POSDeviceAuthenticate(AccountActivatedAPIResource):
             data_in_json        = request.get_json()
             activation_code     = data_in_json.get('activation_code')
             device_setting      = POSSetting.get_by_activation_code(activation_code)
+            access_outlet       = Outlet.fetch(outlet_key)
             assigned_outlet_key = device_setting.assigned_outlet_key
             
             logger.debug('device assigned outlet name=%s', device_setting.assigned_outlet_entity.name)
+            logger.debug('access outlet name=%s', access_outlet.name)
             
             if outlet_key == device_setting.assigned_outlet_key:
                 logger.debug('login outlet is same as device setting assigned outlet')
