@@ -13,7 +13,7 @@ from trexmodel.utils.model.model_util import create_db_client
 #from flask.json import jsonify
 from datetime import datetime, timedelta
 from trexapi.decorators.api_decorators import auth_token_required,\
-    outlet_key_required
+    outlet_key_required, device_is_activated
 from trexlib.utils.string_util import is_not_empty
 from trexmodel.models.datastore.customer_models import Customer
 from trexmodel.models.datastore.user_models import User
@@ -46,6 +46,7 @@ logger = logging.getLogger('debug')
 
 @reward_api_bp.route('/reference-code/<reference_code>/read', methods=['GET'])
 @auth_token_required
+@device_is_activated
 def read_reward(reference_code):
     logger.debug('reference_code=%s', reference_code)
     
@@ -93,6 +94,7 @@ def read_reward(reference_code):
 @reward_api_bp.route('/reference-code/<reference_code>/give', methods=['POST'])
 @auth_token_required
 @outlet_key_required
+@device_is_activated
 def give_reward(reference_code):
     
     logger.debug('reference_code=%s', reference_code)
@@ -219,6 +221,7 @@ def give_reward(reference_code):
 @reward_api_bp.route('/reference-code/<reference_code>/redeem', methods=['POST'])
 @auth_token_required
 @outlet_key_required
+@device_is_activated
 def redeem_reward(reference_code):
     
     logger.debug('reference_code=%s', reference_code)
@@ -319,6 +322,7 @@ def redeem_reward(reference_code):
 @reward_api_bp.route('/reference-code/<reference_code>/prepaid-topup', methods=['POST'])
 @auth_token_required
 @outlet_key_required
+@device_is_activated
 def prepaid_topup(reference_code):
     
     logger.debug('reference_code=%s', reference_code)
@@ -383,6 +387,7 @@ def prepaid_topup(reference_code):
 @reward_api_bp.route('/reference-code/<reference_code>/prepaid-redeem', methods=['POST'])
 @auth_token_required
 @outlet_key_required
+@device_is_activated
 def prepaid_redeem(reference_code):
     
     logger.debug('reference_code=%s', reference_code)
@@ -444,6 +449,7 @@ def prepaid_redeem(reference_code):
 @reward_api_bp.route('/reference-code/<reference_code>/point-redeem', methods=['POST'])
 @auth_token_required
 @outlet_key_required
+@device_is_activated
 def point_redeem(reference_code):
     
     logger.debug('reference_code=%s', reference_code)
