@@ -135,7 +135,9 @@ def post_sales_transaction():
                         entitled_url                = EARN_INSTANT_REWARD_URL.format(code=encrypted_transaction_id)
                         
                         transaction_details =  {
-                                                "entitled_url"            : entitled_url,
+                                                "entitled_url"      : entitled_url,
+                                                "sales_amount"      : str(transaction_form.sales_amount.data),
+                                                "invoice_id"        : invoice_id,
                                                 }
                             
                     return jsonify(transaction_details)
@@ -155,7 +157,7 @@ def post_sales_transaction():
 @sales_api_bp.route('/transaction/<encrypted_transaction_id>/read-instant-reward', methods=['GET'])
 #@user_auth_token_required
 def read_instant_reward_get(encrypted_transaction_id): 
-    
+    logger.debug('encrypted_transaction_id=%s', encrypted_transaction_id)
     try:
         transaction_id    = decrypt(encrypted_transaction_id)
     except:
@@ -211,17 +213,17 @@ def encrypt_instant_reward_code_get(transaction_id):
     
     
 @sales_api_bp.route('/reference-code/<reference_code>/transaction/<encrypted_transaction_id>/customer-earn-instant-reward', methods=['POST'])
-#@user_auth_token_required
+@user_auth_token_required
 def customer_earn_instant_reward(reference_code, encrypted_transaction_id): 
     
     db_client = create_db_client(caller_info="customer_earn_instant_reward")
         
-    logger.debug('encrypted_transaction_id=%s', encrypted_transaction_id)
+    logger.info('encrypted_transaction_id=%s', encrypted_transaction_id)
     
     transaction_id = decrypt(encrypted_transaction_id)
     
-    logger.debug('reference_code=%s', reference_code)
-    logger.debug('transaction_id=%s', transaction_id)
+    logger.info('reference_code=%s', reference_code)
+    logger.info('transaction_id=%s', transaction_id)
         
     if is_not_empty(transaction_id):
         with db_client.context():
@@ -230,7 +232,7 @@ def customer_earn_instant_reward(reference_code, encrypted_transaction_id):
         
         if sales_transaction:
             
-            logger.debug('sales_transaction is found')
+            logger.info('sales_transaction is found')
             if sales_transaction.used:
                 with db_client.context():
                     customer_transaction = CustomerTransaction.get_by_transaction_id(transaction_id)
@@ -243,10 +245,21 @@ def customer_earn_instant_reward(reference_code, encrypted_transaction_id):
                         merchant_acct = sales_transaction.transact_merchant_acct
                         customer = Customer.get_by_reference_code(reference_code, merchant_acct)
                         
+                        if customer is None:
+                            logger.info('Customer is not found, thus going to create new customer account')
+                            instant_reward_issued_outlet = sales_transaction.transact_outlet_entity
+                            
+                            reference_code  = request.headers.get('x-reference-code')
+                            
+                            existing_user_acct = User.get_by_reference_code(reference_code)
+                            
+                            customer = Customer.create_from_user(instant_reward_issued_outlet, existing_user_acct)
+                        
                         if customer:
                             customer_transaction = give_reward_from_sales_transaction(customer, sales_transaction)
                             if customer_transaction:
                                 customer_transaction = customer_transaction.to_dict(date_format="%d-%m-%Y", datetime_format="%d-%m-%Y %H:%M:%S")
+                    
                     if customer:
                         if customer_transaction:
                             return jsonify(customer_transaction)
