@@ -9,7 +9,8 @@ import logging
 from trexlib.utils.log_util import get_tracelog
 from trexmodel.utils.model.model_util import create_db_client
 from datetime import datetime, timedelta
-from trexlib.utils.string_util import is_not_empty, random_number, random_string
+from trexlib.utils.string_util import is_not_empty, random_number, random_string,\
+    is_empty
 from trexmodel.models.datastore.user_models import User
 from trexadmin.libs.http import create_rest_message
 from trexadmin.libs.http import StatusCode
@@ -29,11 +30,14 @@ from flask.json import jsonify
 from trexlib.conf import PRODUCTION_MODE, DEPLOYMENT_MODE, DEMO_MODE, LOCAL_MODE
 from flask_babel import gettext
 from trexadmin import conf
+from trexmodel import conf as model_conf
 import os
 from trexlib.utils.sms_util import send_sms
 from trexmodel.models.datastore.message_models import Message
 from time import strftime
 from trexlib.utils.common.date_util import from_utc_datetime_to_local_datetime
+from trexmodel.conf import USER_STATUS_REGISTERED, GENDER_UNKNOWN_CODE
+from trexmodel.models.datastore.model_decorators import model_transactional
 
 user_api_bp = Blueprint('user_api_bp', __name__,
                                  template_folder='templates',
@@ -111,8 +115,15 @@ def user_register():
                 birth_date      = register_user_form.birth_date.data
                 gender          = register_user_form.gender.data
                 password        = register_user_form.password.data
+                status          = register_user_form.status.data
                 
-                birth_date = datetime.strptime(birth_date, '%d-%m-%Y')
+                if is_not_empty(birth_date):
+                    birth_date = datetime.strptime(birth_date, '%d-%m-%Y')
+                else:
+                    birth_date = None
+                    
+                if is_empty(gender):
+                    gender = model_conf.GENDER_UNKNOWN_CODE
                 
                 logger.debug('email=%s', email)
                 logger.debug('name=%s', name)
@@ -120,32 +131,37 @@ def user_register():
                 logger.debug('birth_date=%s', birth_date)
                 logger.debug('gender=%s', gender)
                 logger.debug('password=%s', password)
+                logger.debug('status=%s', status)
                 
                 checking_registered_user_acct = User.get_by_email(email)
                 if checking_registered_user_acct is None:
                     if is_not_empty(mobile_phone):
                         checking_registered_user_acct = User.get_by_mobile_phone(mobile_phone)
                         if checking_registered_user_acct is None:
-                            registered_user_acct = User.create(name=name, 
-                                                               email=email, 
-                                                               mobile_phone=mobile_phone, 
-                                                               birth_date=birth_date,
-                                                               gender=gender,
-                                                               password=password, 
-                                                               is_email_verified=True, 
-                                                               is_mobile_phone_verified=True
+                            registered_user_acct = User.create(
+                                                                name                        = name, 
+                                                                email                       = email, 
+                                                                mobile_phone                = mobile_phone, 
+                                                                birth_date                  = birth_date,
+                                                                gender                      = gender,
+                                                                password                    = password, 
+                                                                is_email_verified           = True, 
+                                                                is_mobile_phone_verified    = True,
+                                                                status                      = USER_STATUS_REGISTERED,
                                                                )
                             logger.debug('new registered_user_acct=%s', registered_user_acct)
                         else:
                             if checking_registered_user_acct.is_mobile_phone_verified==True:
                                 return create_rest_message('Mobile Phone have been taken', status_code=StatusCode.BAD_REQUEST)
                     else:
-                        registered_user_acct = User.create(name=name, 
-                                                               email=email, 
-                                                               birth_date=birth_date,
-                                                               gender=gender,
-                                                               password=password, 
-                                                               is_email_verified=True,
+                        registered_user_acct = User.create(
+                                                                name                = name, 
+                                                                email               = email, 
+                                                                birth_date          = birth_date,
+                                                                gender              = gender,
+                                                                password            = password, 
+                                                                is_email_verified   = True,
+                                                                status              = USER_STATUS_REGISTERED,
                                                                )
                         
                         logger.debug('new registered_user_acct=%s', registered_user_acct)
@@ -163,6 +179,7 @@ def user_register():
                                            reference_code                       = registered_user_acct.reference_code,
                                            email_vc_expiry_datetime             = registered_user_acct.email_vc_expiry_datetime.strftime('%d-%m-%Y %H:%M:%S') if registered_user_acct.email_vc_expiry_datetime is not None else None,
                                            mobile_phone_vc_expiry_datetime      = registered_user_acct.mobile_phone_vc_expiry_datetime.strftime('%d-%m-%Y %H:%M:%S') if registered_user_acct.mobile_phone_vc_expiry_datetime is not None else None,
+                                           status                               = registered_user_acct.status,
                                            )
             else:
                 return create_rest_message(status_code=StatusCode.BAD_REQUEST)
@@ -197,14 +214,23 @@ def user_update():
                 mobile_phone    = update_user_form.mobile_phone.data
                 birth_date      = update_user_form.birth_date.data
                 gender          = update_user_form.gender.data
+                status          = update_user_form.status.data
+                
+                if is_not_empty(birth_date):
+                    birth_date = datetime.strptime(birth_date, '%d-%m-%Y')
+                else:
+                    birth_date = None
+                    
+                if is_empty(gender):
+                    gender = GENDER_UNKNOWN_CODE    
                 
-                birth_date      = datetime.strptime(birth_date, '%d-%m-%Y')
                 
                 logger.debug('reference_code=%s', reference_code)
                 logger.debug('name=%s', name)
                 logger.debug('mobile_phone=%s', mobile_phone)
                 logger.debug('birth_date=%s', birth_date)
                 logger.debug('gender=%s', gender)
+                logger.debug('status=%s', status)
                 
                 user_acct = User.get_by_reference_code(reference_code)
                 if user_acct:
@@ -216,39 +242,41 @@ def user_update():
                         if original_mobile_phone!=mobile_phone:
                             logger.debug('mobile phone have been changed thus going to check whether new mobile phone is taken or not')
                             checking_mobile_phone_user_acct = User.get_by_mobile_phone(mobile_phone)
+                            
                             if checking_mobile_phone_user_acct is None:
                                 logger.debug('mobile phone is not taken')
                                 
-                                User.update(user_acct, 
+                                _update_user_acct(user_acct, 
                                             mobile_phone    = mobile_phone,
                                             name            = name,
                                             birth_date      = birth_date,
                                             gender          = gender,
-                                            
-                                            )
+                                            status          = status,
+                                            ) 
+                                
                             else:
                                 return create_rest_message('Mobile Phone have been taken', status_code=StatusCode.BAD_REQUEST)
                         else:
                             logger.debug('mobile phone is not change')
                             
-                            User.update(user_acct, 
+                            _update_user_acct(user_acct, 
                                             mobile_phone    = mobile_phone,
                                             name            = name,
                                             birth_date      = birth_date,
                                             gender          = gender,
-                                            
-                                            )
+                                            status          = status,
+                                            ) 
                     
                     else:
                         logger.debug('going to update user details without mobile phone')
                         
-                        User.update(user_acct, 
+                        _update_user_acct(user_acct, 
                                             mobile_phone    = mobile_phone,
                                             name            = name,
                                             birth_date      = birth_date,
                                             gender          = gender,
-                                            
-                                            )          
+                                            status          = status,
+                                            )         
                     
                 else:
                     return create_rest_message('User account is not found', status_code=StatusCode.BAD_REQUEST)
@@ -266,8 +294,46 @@ def user_update():
         logger.error('user_register: Fail to update user due to %s', get_tracelog())
         
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)
-    
 
+@model_transactional(desc="_update_user_account")
+def _update_user_acct(user_acct, mobile_phone=None, name=None, birth_date=None, gender=None, status=None):
+    User.update(user_acct, 
+                        mobile_phone    = mobile_phone,
+                        name            = name,
+                        birth_date      = birth_date,
+                        gender          = gender,
+                        status          = status,
+                        )  
+    Customer.update_all_from_user_acct(user_acct)  
+    
+    
+@user_api_bp.route('/update-status', methods=['PUT'])
+def user_update_status():
+    logger.debug('---user_update_status---')
+    
+    reference_code      = request.headers.get('x-reference-code')
+    
+    try:
+        user_data_in_json   = request.get_json()
+        logger.debug('user_update_status: user_data_in_json=%s', user_data_in_json)
+        
+        update_user_status_form  = UserStatusForm(ImmutableMultiDict(user_data_in_json))
+        if update_user_status_form.validate():
+            logger.debug('update_user_status_form:  update input is valid')
+            db_client = create_db_client(caller_info="user_update_status")
+            
+            with db_client.context():
+                user = User.get_by_reference_code(reference_code)
+                user.status = update_user_status_form.status.data
+                user.put()
+                
+            return create_rest_message(status_code=StatusCode.ACCEPTED)
+
+    except:
+        logger.error('Fail to update user status due to %s', get_tracelog())
+        
+        return create_rest_message(status_code=StatusCode.BAD_REQUEST)
+        
 @user_api_bp.route('/email-auth', methods=['POST'])
 def auth_user_thru_email():
     
@@ -289,8 +355,8 @@ def auth_user_thru_email():
             logger.debug('auth_user: found user account by email=%s', email)    
             logger.debug('auth_user: found user account by password=%s', password)
             
-            if user_acct.locked:
-                return create_rest_message('User account is locked after many trials for security purpose', status_code=StatusCode.BAD_REQUEST)
+            if user_acct.is_still_lock:
+                return create_rest_message('User account is locked after many trials for security purpose. Please try after an hour', status_code=StatusCode.BAD_REQUEST)
             else:
                 if user_acct.deleted:
                     return create_rest_message('User email or password is invalid', status_code=StatusCode.BAD_REQUEST)
@@ -347,8 +413,8 @@ def auth_user_thru_mobile_phone():
             logger.debug('auth_user: found user account by mobile_phone=%s', mobile_phone)    
             logger.debug('auth_user: found user account by password=%s', password)
             
-            if user_acct.locked:
-                return create_rest_message('User account is locked after many trials for security purpose', status_code=StatusCode.BAD_REQUEST)
+            if user_acct.is_still_lock:
+                return create_rest_message('User account is locked after many trials for security purpose. Please try after an hour', status_code=StatusCode.BAD_REQUEST)
             else:
                 if user_acct.deleted:
                     return create_rest_message('User mobile phone or password is invalid', status_code=StatusCode.BAD_REQUEST)
@@ -653,13 +719,18 @@ def user_details_dict(user_acct):
                        'reference_code'                       : user_acct.reference_code, 
                        'name'                                 : user_acct.name, 
                        'email'                                : user_acct.email,
-                       'mobile_phone'                         : user_acct.mobile_phone, 
                        'gender'                               : user_acct.gender,
-                       'birth_date'                           : birth_date_str,  
                        'is_email_verified'                    : is_email_verified,
                        'is_mobile_phone_verified'             : is_mobile_phone_verified,
+                       'status'                               : user_acct.status,
                     }
     
+    if is_not_empty(user_acct.mobile_phone):
+        user_details['mobile_phone'] = user_acct.mobile_phone
+        
+    if is_not_empty(birth_date_str):
+        user_details['birth_date'] = birth_date_str    
+    
     if is_not_empty(email_vc_expiry_datetime):
         user_details['email_vc_expiry_datetime'] = email_vc_expiry_datetime
         
@@ -807,7 +878,7 @@ def read_user_acct(reference_code):
             user_acct = User.get_by_reference_code(reference_code)
         
         if user_acct:
-            logger.debug('verify_user: found user account by reference_code=%s', reference_code)    
+            logger.debug('read_user_acct: found user account by reference_code=%s', reference_code)    
             is_email_verified           = user_acct.is_email_verified
             is_mobile_phone_verified    = user_acct.is_email_verified
             
@@ -831,6 +902,7 @@ def read_user_acct(reference_code):
                                        is_mobile_phone_verified             = is_mobile_phone_verified,
                                        email_vc_expiry_datetime             = email_vc_expiry_datetime,
                                        mobile_phone_vc_expiry_datetime      = mobile_phone_vc_expiry_datetime,
+                                       status                               = user_acct.status,
                                        )
         else:
             logger.debug('user account is not found')
@@ -838,7 +910,45 @@ def read_user_acct(reference_code):
             
     else:
         logger.warn('verify_user: user verify input is invalid')
-        return create_rest_message(status_code=StatusCode.BAD_REQUEST)  
+        return create_rest_message(status_code=StatusCode.BAD_REQUEST) 
+    
+@user_api_bp.route('/delete-account-request', methods=['DELETE'])
+@user_auth_token_required
+def delete_user_acct():
+    reference_code = request.headers.get('x-reference-code')
+    
+    
+    db_client = create_db_client(caller_info="delete_user_acct")
+    
+    password   = request.args.get('password') or request.form.get('password') or request.json.get('password')
+    
+    with db_client.context():
+        user_acct = User.get_by_reference_code(reference_code)
+    
+    if user_acct:
+        if user_acct.is_still_lock:
+            return create_rest_message('User account is locked after many trials for security purpose. Please try after an hour.', status_code=StatusCode.BAD_REQUEST)
+        else:
+            if user_acct.is_valid_password(password):
+                logger.debug('delete_user_acct: found user account by reference_code=%s', reference_code)
+                if user_acct.demo_account==True:
+                    return create_rest_message('Demo account is not allow to delete', status_code=StatusCode.BAD_REQUEST)
+                else:
+                    with db_client.context():
+                        user_acct.request_to_delete() 
+                    
+                    return create_rest_message(status_code=StatusCode.ACCEPTED,)
+            else:
+                with db_client.context():
+                    user_acct.add_try_count()
+                
+                return create_rest_message('Password is not match', status_code=StatusCode.BAD_REQUEST)
+    
+    else:
+        logger.debug('user account is not found')
+        return create_rest_message(status_code=StatusCode.BAD_REQUEST)
+            
+          
     
 @user_api_bp.route('/read-user-new-message-count', methods=['GET'])
 @user_auth_token_required
@@ -996,6 +1106,8 @@ def reset_email_verification_code():
                                        )
         else:
             reference_code = request.headers.get('x-reference-code')
+            logger.debug('reference_code from header=%s', reference_code)
+            logger.debug('reference_code from database=%s', user_acct.reference_code)
             
             if reference_code == user_acct.reference_code:
                 email_vc_expiry_datetime    = _generate_email_expiry_datetime()
