@@ -168,14 +168,15 @@ def redeem_voucher():
             @model_transactional(desc='redeem_voucher')
             def __start_transaction_for_redeem_voucher():
                 
-                customer_redemption = CustomerRedemption.create(customer, program_conf.REWARD_FORMAT_VOUCHER , redeemed_by_outlet,
-                                  redeemed_amount               = 1,            
-                                  redeemed_voucher_keys_list    = to_redeem_voucher_keys_list, 
-                                  redeemed_by                   = redeemed_by, 
-                                  redeemed_datetime             = redeemed_datetime,
-                                  invoice_id                    = invoice_id,
-                                  remarks                       = remarks,
-                                  )
+                customer_redemption = CustomerRedemption.create(customer, program_conf.REWARD_FORMAT_VOUCHER , 
+                                                          redeemed_outlet               = redeemed_by_outlet,
+                                                          redeemed_amount               = 1,            
+                                                          redeemed_voucher_keys_list    = to_redeem_voucher_keys_list, 
+                                                          redeemed_by                   = redeemed_by, 
+                                                          redeemed_datetime             = redeemed_datetime,
+                                                          invoice_id                    = invoice_id,
+                                                          remarks                       = remarks,
+                                                          )
                 return customer_redemption
             
             if to_redeem_voucher_keys_list:
