@@ -129,7 +129,7 @@ def device_is_activated(f):
         if is_not_empty(activation_code) and is_not_empty(device_id):
             device_settings = None
             try:
-                db_client = create_db_client(caller_info="search_customer")
+                db_client = create_db_client(caller_info="device_is_activated")
                 if 'loyalty' == device_type:
                     with db_client.context():
                         device_settings = LoyaltyDeviceSetting.get_by_activation_code(activation_code)
@@ -138,8 +138,12 @@ def device_is_activated(f):
                         device_settings = POSSetting.get_by_activation_code(activation_code)
                 
                 if device_settings is not None:
-                    if device_settings.device_id != device_id:
-                        return create_rest_message('Device is not activated or it is deactivated already', status_code=StatusCode.BAD_REQUEST)
+                    if device_settings.testing:
+                        logger.info('Activation device id is meant for testing thus, it can be by pass')
+                    else:
+                        if device_settings.device_id != device_id:
+                            logger.info('Access device id is not same as activation device id')
+                            return create_rest_message('Device is not activated or it is deactivated already', status_code=StatusCode.BAD_REQUEST)
                     
                     return f(*args, **kwargs)
                 
@@ -187,3 +191,20 @@ def merchant_key_required(f):
         return create_rest_message('Missing merchant account id', status_code=StatusCode.UNAUTHORIZED)
 
     return decorated_function
+
+def customer_key_required(f):
+    @wraps(f)
+    def decorated_function(*args, **kwargs):
+        customer_key = request.headers.get('x-customer-key')
+            
+        logger.debug('customer_key=%s', customer_key)
+        
+        if is_not_empty(customer_key):
+            logger.debug('Going to execute')
+            return f(customer_key, **kwargs)
+            
+        
+        #return ("Outlet Key is required", 401)
+        return create_rest_message('Missing customer key', status_code=StatusCode.UNAUTHORIZED)
+
+    return decorated_function
\ No newline at end of file
