@@ -9,8 +9,9 @@ from trexadmin.forms.base_forms import ValidationBaseForm
 from trexadmin.libs.wtforms import validators as custom_validator
 from trexadmin.libs.wtforms.fields import OptionalDateTimeField, CurrencyField, JSONField
 from flask_babel import gettext
+from trexapi.forms.sales_api_forms import SalesTransactionForm
 
-class GiveRewardTransactionForm(ValidationBaseForm):
+class GiveRewardTransactionForm(SalesTransactionForm):
     sales_amount                    = DecimalField('Sales Amount',[
                                                 validators.InputRequired(message="Sales Amount is required"),
                                             ])
