@@ -69,7 +69,7 @@ def create_sales_transaction(transact_outlet=None, sales_amount=.0, tax_amount=.
                                        
                                        )
             
-        create_merchant_sales_transaction_upstream_for_merchant(sales_transaction, streamed_datetime=transact_datetime)
+        #create_merchant_sales_transaction_upstream_for_merchant(sales_transaction, streamed_datetime=transact_datetime)
         
         return sales_transaction
         
@@ -620,7 +620,8 @@ def redeem_reward_transaction(customer, redeem_outlet=None, reward_format=None,
             
             logger.debug('reward_format=%s', reward_format)
             
-            customer_redemption = CustomerRedemption.create(customer, reward_format, redeem_outlet, 
+            customer_redemption = CustomerRedemption.create(customer, reward_format, 
+                                      redeemed_outlet               = redeem_outlet, 
                                       redeemed_amount               = reward_amount,
                                       redeemed_voucher_keys_list    = redeemed_voucher_keys_list,                      
                                       invoice_id                    = invoice_id, 
